<?php

namespace FedExVendor\WPDesk\FedexShippingService\FedexApi;

use FedExVendor\FedEx\RateService\ComplexType\RateReply;
use FedExVendor\FedEx\RateService\SimpleType\RateTypeBasisType;
use FedExVendor\FedEx\RateService\SimpleType\ReturnedRateType;
use FedExVendor\WPDesk\AbstractShipping\Rate\Money;
use FedExVendor\WPDesk\AbstractShipping\Rate\ShipmentRating;
use FedExVendor\WPDesk\AbstractShipping\Rate\SingleRate;
use FedExVendor\WPDesk\FedexShippingService\FedexSettingsDefinition;
/**
 * Get response from API
 *
 * @package WPDesk\FedexShippingService\FedexApi
 */
class FedexRateReplyInterpretation implements \FedExVendor\WPDesk\AbstractShipping\Rate\ShipmentRating
{
    /**
     * Is tax enabled.
     *
     * @var bool
     */
    private $is_tax_enabled;
    /**
     * Reply.
     *
     * @var RateReply
     */
    private $reply;
    /**
     * Setting value of FedexSettingsDefinition::FIELD_REQUEST_TYPE
     *
     * @var string
     */
    private $rate_type;
    /**
     * FedexResponse constructor.
     *
     * @param RateReply $reply Rate reply.
     * @param bool $is_tax_enabled Is tax enabled.
     * @param string $rate_type Setting value of FedexSettingsDefinition::FIELD_REQUEST_TYPE
     */
    public function __construct(\FedExVendor\FedEx\RateService\ComplexType\RateReply $reply, $is_tax_enabled, $rate_type)
    {
        $this->reply = $reply;
        $this->is_tax_enabled = $is_tax_enabled;
        $this->rate_type = $rate_type;
    }
    /**
     * Has reply error.
     *
     * @param RateReply $reply Rate reply.
     *
     * @return bool
     */
    public static function has_reply_error(\FedExVendor\FedEx\RateService\ComplexType\RateReply $reply)
    {
        try {
            return 'ERROR' === $reply->HighestSeverity || 'FAILURE' === $reply->HighestSeverity;
        } catch (\Throwable $e) {
            return \true;
        } catch (\Exception $e) {
            // required fallback from Throwable in PHP 5.6
            return \true;
        }
    }
    /**
     * Get reply error message.
     *
     * @param RateReply $reply Rate reply.
     *
     * @return mixed|string
     */
    public static function get_reply_message(\FedExVendor\FedEx\RateService\ComplexType\RateReply $reply)
    {
        try {
            $notification = $reply->Notifications[0];
            if (\is_string($notification)) {
                return $notification;
            }
            return $notification->Message;
        } catch (\Throwable $e) {
            return '';
        } catch (\Exception $e) {
            // required fallback from Throwable in PHP 5.6
            return '';
        }
    }
    /**
     * Has reply warning.
     *
     * @param RateReply $reply Rate reply.
     *
     * @return bool
     */
    public static function has_reply_warning(\FedExVendor\FedEx\RateService\ComplexType\RateReply $reply)
    {
        return 'WARNING' === $reply->HighestSeverity;
    }
    /**
     * Get response from FedEx.
     *
     * @return SingleRate[]
     */
    public function get_ratings()
    {
        $rates = [];
        if (!empty($this->reply->RateReplyDetails)) {
            //phpcs:ignore
            foreach ($this->reply->RateReplyDetails as $reply_detail) {
                if (!empty($reply_detail->RatedShipmentDetails)) {
                    foreach ($reply_detail->RatedShipmentDetails as $ratedShipmentDetail) {
                        if ($this->rate_type === \FedExVendor\FedEx\RateService\SimpleType\RateTypeBasisType::_LIST) {
                            // LIST quotes return both ACCOUNT rates (in RatedShipmentDetails[1])
                            // and LIST rates (in RatedShipmentDetails[3])
                            if (\false === \strpos($ratedShipmentDetail->ShipmentRateDetail->RateType, 'PAYOR_LIST')) {
                                break;
                            }
                        }
                        $rate = new \FedExVendor\WPDesk\AbstractShipping\Rate\SingleRate();
                        $money = new \FedExVendor\WPDesk\AbstractShipping\Rate\Money();
                        if ($this->is_tax_enabled) {
                            $money->currency = $ratedShipmentDetail->ShipmentRateDetail->TotalNetFedExCharge->Currency;
                            $money->amount = $ratedShipmentDetail->ShipmentRateDetail->TotalNetFedExCharge->Amount;
                        } else {
                            $money->currency = $ratedShipmentDetail->ShipmentRateDetail->TotalNetCharge->Currency;
                            $money->amount = $ratedShipmentDetail->ShipmentRateDetail->TotalNetCharge->Amount;
                        }
                        $rate->total_charge = $money;
                        $rate->service_type = $reply_detail->ServiceType;
                        $rate->service_name = $reply_detail->ServiceDescription->Description;
                        if (!empty($reply_detail->ServiceType)) {
                            $rates[] = $rate;
                        }
                    }
                }
            }
        }
        return $rates;
    }
}
