<?php

namespace FedExVendor\WPDesk\FedexShippingService;

use Psr\Log\LoggerInterface;
use FedExVendor\WPDesk\AbstractShipping\Exception\UnitConversionException;
use FedExVendor\WPDesk\AbstractShipping\ShippingServiceCapability\CanInsure;
use FedExVendor\WPDesk\AbstractShipping\ShippingServiceCapability\CanPack;
use FedExVendor\WPDesk\AbstractShipping\Shop\ShopSettings;
use FedExVendor\WPDesk\FedexShippingService\Exception\CurrencySwitcherException;
use FedExVendor\WPDesk\AbstractShipping\Exception\InvalidSettingsException;
use FedExVendor\WPDesk\AbstractShipping\Exception\RateException;
use FedExVendor\WPDesk\AbstractShipping\Rate\ShipmentRating;
use FedExVendor\WPDesk\AbstractShipping\Rate\ShipmentRatingImplementation;
use FedExVendor\WPDesk\AbstractShipping\Settings\SettingsValues;
use FedExVendor\WPDesk\AbstractShipping\Shipment\Shipment;
use FedExVendor\WPDesk\AbstractShipping\ShippingService;
use FedExVendor\WPDesk\AbstractShipping\ShippingServiceCapability\CanRate;
use FedExVendor\WPDesk\AbstractShipping\ShippingServiceCapability\HasSettings;
use FedExVendor\WPDesk\FedexShippingService\FedexApi\FedexRateCustomServicesFilter;
use FedExVendor\WPDesk\FedexShippingService\FedexApi\FedexRateReplyInterpretation;
use FedExVendor\WPDesk\FedexShippingService\FedexApi\FedexRateRequestBuilder;
use FedExVendor\WPDesk\FedexShippingService\FedexApi\Sender;
use FedExVendor\WPDesk\FedexShippingService\FedexApi\FedexSender;
/**
 * FedEx main shipping class injected into WooCommerce shipping method.
 *
 * @package WPDesk\FedexShippingService
 */
class FedexShippingService extends \FedExVendor\WPDesk\AbstractShipping\ShippingService implements \FedExVendor\WPDesk\AbstractShipping\ShippingServiceCapability\HasSettings, \FedExVendor\WPDesk\AbstractShipping\ShippingServiceCapability\CanRate, \FedExVendor\WPDesk\AbstractShipping\ShippingServiceCapability\CanInsure, \FedExVendor\WPDesk\AbstractShipping\ShippingServiceCapability\CanPack
{
    /** Logger.
     *
     * @var LoggerInterface
     */
    private $logger;
    /** Shipping method helper.
     *
     * @var ShopSettings
     */
    private $shop_settings;
    const UNIQUE_ID = 'flexible_shipping_fedex';
    /**
     * Sender.
     *
     * @var Sender
     */
    private $sender;
    /**
     * FedexShippingService constructor.
     *
     * @param LoggerInterface $logger Logger.
     * @param ShopSettings $helper Helper.
     */
    public function __construct(\Psr\Log\LoggerInterface $logger, \FedExVendor\WPDesk\AbstractShipping\Shop\ShopSettings $helper)
    {
        $this->logger = $logger;
        $this->shop_settings = $helper;
    }
    public function is_rate_enabled(\FedExVendor\WPDesk\AbstractShipping\Settings\SettingsValues $settings)
    {
        return \true;
    }
    /**
     * Set logger.
     *
     * @param LoggerInterface $logger Logger.
     */
    public function setLogger(\Psr\Log\LoggerInterface $logger)
    {
        $this->logger = $logger;
    }
    /**
     * Set sender.
     *
     * @param Sender $sender Sender.
     */
    public function set_sender(\FedExVendor\WPDesk\FedexShippingService\FedexApi\Sender $sender)
    {
        $this->sender = $sender;
    }
    /**
     * Get sender.
     *
     * @return Sender
     */
    public function get_sender()
    {
        return $this->sender;
    }
    /**
     * Rate shipment.
     *
     * @param SettingsValues $settings Settings Values.
     * @param Shipment $shipment Shipment.
     *
     * @return ShipmentRating
     * @throws InvalidSettingsException InvalidSettingsException.
     * @throws RateException RateException.
     * @throws UnitConversionException Weight exception.
     */
    public function rate_shipment(\FedExVendor\WPDesk\AbstractShipping\Settings\SettingsValues $settings, \FedExVendor\WPDesk\AbstractShipping\Shipment\Shipment $shipment)
    {
        if (!$this->get_settings_definition()->validate_settings($settings)) {
            throw new \FedExVendor\WPDesk\AbstractShipping\Exception\InvalidSettingsException();
        }
        $validate_shipment = new \FedExVendor\WPDesk\FedexShippingService\FedexValidateShipment($shipment, $this->logger);
        $this->verify_currency($this->shop_settings->get_default_currency(), $this->shop_settings->get_currency());
        if ($validate_shipment->is_weight_exceeded()) {
            return new \FedExVendor\WPDesk\AbstractShipping\Rate\ShipmentRatingImplementation([]);
        }
        $request_builder = new \FedExVendor\WPDesk\FedexShippingService\FedexApi\FedexRateRequestBuilder($settings, $shipment, $this->shop_settings);
        $request = $request_builder->build_request();
        $this->set_sender(new \FedExVendor\WPDesk\FedexShippingService\FedexApi\FedexSender($this->logger, $this->is_testing($settings)));
        $response = $this->get_sender()->send($request);
        $reply = new \FedExVendor\WPDesk\FedexShippingService\FedexApi\FedexRateReplyInterpretation($response, $this->shop_settings->is_tax_enabled(), $settings->get_value(\FedExVendor\WPDesk\FedexShippingService\FedexSettingsDefinition::FIELD_REQUEST_TYPE, \FedExVendor\WPDesk\FedexShippingService\FedexSettingsDefinition::FIELD_REQUEST_TYPE_VALUE_ALL));
        if ($reply::has_reply_warning($response)) {
            $this->logger->info($reply::get_reply_message($response));
        }
        return new \FedExVendor\WPDesk\FedexShippingService\FedexApi\FedexRateCustomServicesFilter($reply, $settings);
    }
    /**
     * Verify currency.
     *
     * @param string $default_shop_currency Shop currency.
     * @param string $checkout_currency Checkout currency.
     *
     * @return void
     * @throws CurrencySwitcherException .
     */
    private function verify_currency($default_shop_currency, $checkout_currency)
    {
        if ($default_shop_currency !== $checkout_currency) {
            throw new \FedExVendor\WPDesk\FedexShippingService\Exception\CurrencySwitcherException();
        }
    }
    /**
     * Should I use a test API?
     *
     * @param \WPDesk\AbstractShipping\Settings\SettingsValues $settings Settings.
     *
     * @return bool
     */
    private function is_testing(\FedExVendor\WPDesk\AbstractShipping\Settings\SettingsValues $settings)
    {
        $testing = \false;
        if ($settings->has_value('testing') && $this->shop_settings->is_testing()) {
            $testing = 'yes' === $settings->get_value('testing') ? \true : \false;
        }
        return $testing;
    }
    /**
     * Get settings
     *
     * @return FedexSettingsDefinition
     */
    public function get_settings_definition()
    {
        return new \FedExVendor\WPDesk\FedexShippingService\FedexSettingsDefinition($this->shop_settings);
    }
    /**
     * Get unique ID.
     *
     * @return string
     */
    public function get_unique_id()
    {
        return self::UNIQUE_ID;
    }
    /**
     * Get name.
     *
     * @return string
     */
    public function get_name()
    {
        return \__('FedEx', 'flexible-shipping-fedex');
    }
    /**
     * Get description.
     *
     * @return string
     */
    public function get_description()
    {
        return \__('FedEx integration', 'flexible-shipping-fedex');
    }
}
