<?php

namespace FedExVendor\WPDesk\FedexShippingService;

use Psr\Log\LoggerInterface;
use FedExVendor\WPDesk\AbstractShipping\Exception\UnitConversionException;
use FedExVendor\WPDesk\AbstractShipping\Shipment\Shipment;
use FedExVendor\WPDesk\AbstractShipping\Shipment\Weight;
use FedExVendor\WPDesk\AbstractShipping\UnitConversion\UniversalWeight;
use FedExVendor\WPDesk\FedexShippingService\FedexApi\FedexRequestManipulation;
/**
 * Validate shipment for some cases.
 *
 * @package WPDesk\FedexShippingService
 */
class FedexValidateShipment
{
    /**
     * Maximum weights for FedEx shipping.
     *
     * @var array
     */
    const FEDEX_MAX_WEIGHTS = [\FedExVendor\WPDesk\AbstractShipping\Shipment\Weight::WEIGHT_UNIT_G => 68000, \FedExVendor\WPDesk\AbstractShipping\Shipment\Weight::WEIGHT_UNIT_KG => 68, \FedExVendor\WPDesk\AbstractShipping\Shipment\Weight::WEIGHT_UNIT_LB => 150, \FedExVendor\WPDesk\AbstractShipping\Shipment\Weight::WEIGHT_UNIT_OZ => 2433];
    /**
     * Shipment.
     *
     * @var \WPDesk\AbstractShipping\Shipment\Shipment
     */
    private $shipment;
    /**
     * Logger.
     *
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;
    /**
     * FedexValidateShipment constructor.
     *
     * @param \WPDesk\AbstractShipping\Shipment\Shipment $shipment Shipment.
     * @param \Psr\Log\LoggerInterface $logger Logger.
     */
    public function __construct(\FedExVendor\WPDesk\AbstractShipping\Shipment\Shipment $shipment, \Psr\Log\LoggerInterface $logger)
    {
        $this->shipment = $shipment;
        $this->logger = $logger;
    }
    /**
     * Is weight exceeded.
     *
     * @return bool
     * @throws UnitConversionException Weight exception.
     */
    public function is_weight_exceeded()
    {
        $item_unit = \FedExVendor\WPDesk\AbstractShipping\Shipment\Weight::WEIGHT_UNIT_KG;
        $total_weight = 0;
        foreach ($this->shipment->packages as $package) {
            foreach ($package->items as $item) {
                $item_unit = \FedExVendor\WPDesk\FedexShippingService\FedexApi\FedexRequestManipulation::convert_weight_unit($item->weight->weight_unit);
                $total_weight += (new \FedExVendor\WPDesk\AbstractShipping\UnitConversion\UniversalWeight($item->weight->weight, $item_unit))->as_unit_rounded(\FedExVendor\WPDesk\AbstractShipping\Shipment\Weight::WEIGHT_UNIT_KG);
            }
        }
        if ($total_weight > self::FEDEX_MAX_WEIGHTS[\FedExVendor\WPDesk\AbstractShipping\Shipment\Weight::WEIGHT_UNIT_KG]) {
            $notice = \sprintf(
                // translators: %1$s weight unit. %2$s number of max weight in kg or lb.
                \__('FedEx: The maximum weight for %1$s unit has has been exceeded. Maximum weight is: %2$s.', 'flexible-shipping-fedex'),
                $item_unit,
                self::FEDEX_MAX_WEIGHTS[$item_unit]
            );
            $this->logger->error($notice);
            return \true;
        }
        return \false;
    }
}
